# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for `rbd_iscsi_client` package."""

import mock
import requests
import unittest

from rbd_iscsi_client import client
from rbd_iscsi_client import exceptions


class TestRbd_iscsi_client(unittest.TestCase):
    """Tests for `rbd_iscsi_client` package."""

    client = None
    FAKE_URL = 'client://fake-url:0000'
    FAKE_USER = 'user'
    FAKE_PASSWORD = 'password'

    def setUp(self):
        self.client = client.RBDISCSIClient(self.FAKE_USER,
                                            self.FAKE_PASSWORD,
                                            self.FAKE_URL,
                                            secure=False,
                                            http_log_debug=True,
                                            suppress_ssl_warnings=False,
                                            timeout=None)

    def tearDown(self):
        self.client = None

    def test_request_timeout(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()
        http_method = 'fake this'
        http_url = 'http://fake-url:0000'

        with mock.patch('requests.request', retest, create=True):
            # Test timeout exception
            retest.side_effect = requests.exceptions.Timeout
            self.assertRaises(exceptions.Timeout,
                              self.client.request,
                              http_url, http_method)

    def test_request_redirects(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()
        http_method = 'fake this'
        http_url = 'http://fake-url:0000'

        with mock.patch('requests.request', retest, create=True):
            # Test too many redirects exception
            retest.side_effect = requests.exceptions.TooManyRedirects
            self.assertRaises(exceptions.TooManyRedirects,
                              self.client.request,
                              http_url, http_method)

    def test_request_http_error(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()
        http_method = 'fake this'
        http_url = 'http://fake-url:0000'

        with mock.patch('requests.request', retest, create=True):
            # Test HTTP Error exception
            retest.side_effect = requests.exceptions.HTTPError
            self.assertRaises(exceptions.HTTPError,
                              self.client.request,
                              http_url, http_method)

    def test_request_url_required(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()
        http_method = 'fake this'
        http_url = 'http://fake-url:0000'

        with mock.patch('requests.request', retest, create=True):
            # Test URL required exception
            retest.side_effect = requests.exceptions.URLRequired
            self.assertRaises(exceptions.URLRequired,
                              self.client.request,
                              http_url, http_method)

    def test_request_exception(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()
        http_method = 'fake this'
        http_url = 'http://fake-url:0000'

        with mock.patch('requests.request', retest, create=True):
            # Test request exception
            retest.side_effect = requests.exceptions.RequestException
            self.assertRaises(exceptions.RequestException,
                              self.client.request,
                              http_url, http_method)

    def test_request_ssl_error(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()

        with mock.patch('requests.request', retest, create=True):
            # Test requests exception
            retest.side_effect = requests.exceptions.SSLError
            self.assertRaisesRegexp(exceptions.SSLCertFailed, "failed")

    def test_client_timeout_setting(self):
        self.client._http_log_req = mock.Mock()
        self.client.timeout = 10
        retest = mock.Mock()

        with mock.patch('requests.request', retest, create=True):
            self.assertEqual(self.client.timeout, 10)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for `rbd_iscsi_client` package."""

import unittest

from rbd_iscsi_client import exceptions


class Test_exceptions(unittest.TestCase):
    """Tests for `rbd_iscsi_client` package."""

    def setUp(self):
        """Set up test fixtures, if any."""

    def tearDown(self):
        """Tear down test fixtures, if any."""

    def test_000_from_response_string_format(self):
        """Test something."""
        class FakeResponse(object):
            status = 500

        fake_response = FakeResponse()
        output = exceptions.from_response(fake_response, {}).__str__()
        self.assertEqual('Internal Server Error (HTTP 500)', output)

    def test_001_client_exception_string_format(self):
        fake_error = {'code': 999,
                      'desc': 'Fake Description',
                      'ref': 'Fake Ref',
                      'debug1': 'Fake Debug 1',
                      'debug2': 'Fake Debug 2', }
        client_ex = exceptions.ClientException(error=fake_error)
        client_ex.message = "Fake Error"
        client_ex.http_status = 500
        output = client_ex.__str__()

        self.assertEqual("Fake Error (HTTP 500) 999 - Fake Description - "
                         "Fake Ref (1: 'Fake Debug 1') (2: 'Fake Debug 2')",
                         output)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""Tests for `rbd_iscsi_client` package."""

import mock
import unittest

from rbd_iscsi_client import client


class TestRbd_iscsi_client(unittest.TestCase):
    """Tests for `rbd_iscsi_client` package."""
    FAKE_URL = "http://fake-url:5000"

    RESP_200 = {'status': '200', 'Content-Length': '8787',
                'content-location': u'http://fake-url:5000/api/config',
                'Server': 'Werkzeug/0.14.1 Python/2.7.15rc1',
                'Date': 'Thu, 13 Jun 2019 18:56:46 GMT',
                'Content-Type': 'application/json'}

    def setUp(self):
        """Set up test fixtures, if any."""
        fake_url = 'client://fake-url:0000'
        fake_user = 'user'
        fake_password = 'password'
        self.client = client.RBDISCSIClient(fake_user, fake_password,
                                            fake_url, secure=False,
                                            http_log_debug=True,
                                            suppress_ssl_warnings=False,
                                            timeout=10)
        self.client._http_log_req = mock.Mock()

    def tearDown(self):
        """Tear down test fixtures, if any."""

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_get(self, cs_mock):
        """Test something."""
        fake_uri = '/foo'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.get(fake_uri)
        cs_mock.assert_called_with(fake_uri, 'GET')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_put(self, cs_mock):
        """Test something."""
        fake_uri = '/foo'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.put(fake_uri)
        cs_mock.assert_called_with(fake_uri, 'PUT')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_post(self, cs_mock):
        """Test something."""
        fake_uri = '/foo'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.post(fake_uri)
        cs_mock.assert_called_with(fake_uri, 'POST')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_delete(self, cs_mock):
        """Test something."""
        fake_uri = '/foo'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.delete(fake_uri)
        cs_mock.assert_called_with(fake_uri, 'DELETE')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_get_api(self, cs_mock):
        """Test something."""
        fake_uri = '/api'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.get_api()
        cs_mock.assert_called_with(fake_uri, 'GET')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_get_config(self, cs_mock):
        """Test something."""
        fake_uri = '/api/config'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.get_config()
        cs_mock.assert_called_with(fake_uri, 'GET')
        self.assertEqual(response, self.RESP_200)

    @mock.patch.object(client.RBDISCSIClient, '_cs_request')
    def test_get_gatewayinfo(self, cs_mock):
        """Test something."""
        fake_uri = '/api/gatewayinfo'
        body = {}
        cs_mock.return_value = (self.RESP_200, body)
        response, content = self.client.get_gatewayinfo()
        cs_mock.assert_called_with(fake_uri, 'GET')
        self.assertEqual(response, self.RESP_200)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Exceptions for the client

.. module: exceptions

:Author: Walter A. Boring IV
:Description: This contains the HTTP exceptions from rbd-target-api
"""

import logging

# Python 3+ override
try:
    basestring
except NameError:
    basestring = str

LOG = logging.getLogger(__name__)


class UnsupportedVersion(Exception):
    """unsupported version of the API."""
    pass


class CommandError(Exception):
    pass


class AuthorizationFailure(Exception):
    pass


class NoUniqueMatch(Exception):
    pass


class ClientException(Exception):
    """The base exception class for all exceptions this library raises.

    :param error: The error array
    :type error: array

    """
    _error_code = None
    _error_desc = None
    _error_ref = None

    _debug1 = None
    _debug2 = None

    def __init__(self, error=None):
        super(ClientException, self).__init__()

        if not error:
            return

        if isinstance(error, basestring):
            # instead of KeyError below, take it and make it the _error_desc.
            self._error_desc = error
        else:
            if 'code' in error:
                self._error_code = error['code']
            if 'desc' in error:
                self._error_desc = error['desc']
            if 'ref' in error:
                self._error_ref = error['ref']

            if 'debug1' in error:
                self._debug1 = error['debug1']
            if 'debug2' in error:
                self._debug2 = error['debug2']

    def get_code(self):
        return self._error_code

    def get_description(self):
        return self._error_desc

    def get_ref(self):
        return self._error_ref

    def __str__(self):
        formatted_string = self.message
        if self.http_status:
            formatted_string += " (HTTP %s)" % self.http_status
        if self._error_code:
            formatted_string += " %s" % self._error_code
        if self._error_desc:
            formatted_string += " - %s" % self._error_desc
        if self._error_ref:
            formatted_string += " - %s" % self._error_ref

        if self._debug1:
            formatted_string += " (1: '%s')" % self._debug1

        if self._debug2:
            formatted_string += " (2: '%s')" % self._debug2

        return formatted_string

# SSL Errors


class SSLCertFailed(ClientException):
    """The SSL certificate from the server could not be verified."""
    http_status = ""
    message = "SSL Certificate Verification Failed"


#  Python Requests Errors


class RequestException(ClientException):
    """There was an ambiguous exception that occurred in Requests."""
    pass


class ConnectionError(ClientException):
    """There was an error connecting to the server."""
    pass


class HTTPError(ClientException):
    """An HTTP error occurred."""
    pass


class URLRequired(ClientException):
    """A valid URL is required to make a request."""
    pass


class TooManyRedirects(ClientException):
    """Too many redirects."""
    pass


class Timeout(ClientException):
    """The request timed out."""
    pass


# 400 Errors


class HTTPBadRequest(ClientException):
    """HTTP 400 - Bad request: you sent some malformed data."""
    http_status = 400
    message = "Bad request"


class HTTPUnauthorized(ClientException):
    """HTTP 401 - Unauthorized: bad credentials."""
    http_status = 401
    message = "Unauthorized"


class HTTPForbidden(ClientException):
    """HTTP 403 - Forbidden: credentials don't give access to this resource."""
    http_status = 403
    message = "Forbidden"


class HTTPNotFound(ClientException):
    """HTTP 404 - Not found """
    http_status = 404
    message = "Not found"


class HTTPMethodNotAllowed(ClientException):
    """HTTP 405 - Method not Allowed """
    http_status = 405
    message = "Method Not Allowed"


class HTTPNotAcceptable(ClientException):
    """HTTP 406 - Method not Acceptable """
    http_status = 406
    message = "Method Not Acceptable"


class HTTPProxyAuthRequired(ClientException):
    """HTTP 407 - The client must first authenticate itself with the proxy."""
    http_status = 407
    message = "Proxy Authentication Required"


class HTTPRequestTimeout(ClientException):
    """HTTP 408 - The server timed out waiting for the request."""
    http_status = 408
    message = "Request Timeout"


class HTTPConflict(ClientException):
    """HTTP 409 - Conflict: A Conflict happened on the server."""
    http_status = 409
    message = "Conflict"


class HTTPGone(ClientException):
    """HTTP 410

    Indicates that the resource requested is no longer available and
    will not be available again.
    """
    http_status = 410
    message = "Gone"


class HTTPLengthRequired(ClientException):
    """HTTP 411

    The request did not specify the length of its content, which is
               required by the requested resource.

    """
    http_status = 411
    message = "Length Required"


class HTTPPreconditionFailed(ClientException):
    """HTTP 412

    The server does not meet one of the preconditions that the
               requester put on the request.
    """
    http_status = 412
    message = "Over limit"


class HTTPRequestEntityTooLarge(ClientException):
    """HTTP 413

    The request is larger than the server is willing or able to process
    """
    http_status = 413
    message = "Request Entity Too Large"


class HTTPRequestURITooLong(ClientException):
    """HTTP 414 - The URI provided was too long for the server to process."""
    http_status = 414
    message = "Request URI Too Large"


class HTTPUnsupportedMediaType(ClientException):
    """HTTP 415

    The request entity has a media type which the server or resource
    does not support.
    """

    http_status = 415
    message = "Unsupported Media Type"


class HTTPRequestedRangeNotSatisfiable(ClientException):
    """HTTP 416

    The client has asked for a portion of the file, but the server cannot
    supply that portion.
    """
    http_status = 416
    message = "Requested Range Not Satisfiable"


class HTTPExpectationFailed(ClientException):
    """HTTP 417

    The server cannot meet the requirements of the Expect
    request-header field.
    """

    http_status = 417
    message = "Expectation Failed"


class HTTPTeaPot(ClientException):
    """HTTP 418 - I'm a Tea Pot."""
    http_status = 418
    message = "I'm A Teapot. (RFC 2324)"


# 500 Errors


class HTTPInternalServerError(ClientException):
    """HTTP 500 - Internal Server Error: an internal error occured."""
    http_status = 500
    message = "Internal Server Error"


class HTTPNotImplemented(ClientException):
    """HTTP 501 - Not Implemented: server does not support the operation."""
    http_status = 501
    message = "Not Implemented"


class HTTPBadGateway(ClientException):
    """HTTP 502

    The server was acting as a gateway or proxy and received an
    invalid response from the upstream server.
    """
    http_status = 502
    message = "Bad Gateway"


class HTTPServiceUnavailable(ClientException):
    """HTTP 503 - The server is currently unavailable."""
    http_status = 503
    message = "Service Unavailable"


class HTTPGatewayTimeout(ClientException):
    """HTTP 504

    The server was acting as a gateway or proxy and did
    not receive a timely response from the upstream server.
    """
    http_status = 504
    message = "Gateway Timeout"


class HTTPVersionNotSupported(ClientException):
    """HTTP 505

    The server does not support the HTTP protocol version used in the request.
    """
    http_status = 505
    message = "Version Not Supported"


# In Python 2.4 Exception is old-style and thus doesn't have a __subclasses__()
# so we can do this:
#     _code_map = dict((c.http_status, c)
#                      for c in ClientException.__subclasses__())
#
# Instead, we have to hardcode it:
_code_map = dict((c.http_status, c) for c in
                 [HTTPBadRequest, HTTPUnauthorized,
                  HTTPForbidden, HTTPNotFound, HTTPMethodNotAllowed,
                  HTTPNotAcceptable, HTTPProxyAuthRequired,
                  HTTPRequestTimeout, HTTPConflict, HTTPGone,
                  HTTPLengthRequired, HTTPPreconditionFailed,
                  HTTPRequestEntityTooLarge, HTTPRequestURITooLong,
                  HTTPUnsupportedMediaType, HTTPRequestedRangeNotSatisfiable,
                  HTTPExpectationFailed, HTTPTeaPot,
                  HTTPNotImplemented, HTTPBadGateway,
                  HTTPServiceUnavailable, HTTPGatewayTimeout,
                  HTTPVersionNotSupported, HTTPInternalServerError])


def from_response(response, body):
    """Return an instance of an ClientException or subclass.

    based on a Python Requests response.

    Usage::

        resp, body = http.request(...)
        if resp.status != 200:
            raise exception_from_response(resp, body)

    """
    cls = _code_map.get(response.status, ClientException)
    return cls(body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License."""
""" RBDISCSIClient.

.. module: rbd_iscsi_client

:Author: Walter A. Boring IV
:Description: This is the HTTP REST Client that is used to make calls to
the ceph-iscsi/rbd-target-api service running on the ceph iscsi gateway
host.
"""

import json
import logging
import requests
import time

from rbd_iscsi_client import exceptions


class RBDISCSIClient(object):
    """REST client to rbd-target-api."""

    USER_AGENT = "os_client"

    username = None
    password = None
    api_url = None

    auth = None
    http_log_debug = False
    tries = 5
    delay = 0
    backoff = 2
    timeout = 60

    _logger = logging.getLogger(__name__)
    retry_exceptions = (exceptions.HTTPServiceUnavailable,
                        requests.exceptions.ConnectionError)

    def __init__(self, username, password, base_url,
                 suppress_ssl_warnings=False, timeout=None,
                 secure=False, http_log_debug=False):
        super(RBDISCSIClient, self).__init__()

        self.username = username
        self.password = password
        self.api_url = base_url
        self.timeout = timeout
        self.secure = secure

        self.times = []
        self.set_debug_flag(http_log_debug)

        if suppress_ssl_warnings:
            requests.packages.urllib3.disable_warnings()

        self.auth = requests.auth.HTTPBasicAuth(username, password)

    def set_debug_flag(self, flag):
        """Turn on/off http request/response debugging."""
        if not self.http_log_debug and flag:
            ch = logging.StreamHandler()
            self._logger.setLevel(logging.DEBUG)
            self._logger.addHandler(ch)
            self.http_log_debug = True

    def _http_log_req(self, args, kwargs):
        if not self.http_log_debug:
            return

        string_parts = ['curl -i']
        for element in args:
            if element in ('GET', 'POST'):
                string_parts.append(' -X %s' % element)
            else:
                string_parts.append(' %s' % element)

        for element in kwargs['headers']:
            header = ' -H "%s: %s"' % (element, kwargs['headers'][element])
            string_parts.append(header)

        if 'data' in kwargs:
            string_parts.append(' -d ')
            for key in kwargs['data']:
                string_parts.append('%(key)s=%(value)s&' %
                                    {'key': key,
                                     'value': kwargs['data'][key]})

        self._logger.debug("\nREQ: %s\n" % "".join(string_parts))

    def _http_log_resp(self, resp, body):
        if not self.http_log_debug:
            return
        # Replace commas with newlines to break the debug into new lines,
        # making it easier to read
        self._logger.debug("RESP:%s\n", str(resp).replace("',", "'\n"))
        self._logger.debug("RESP BODY:%s\n", body)

    def request(self, *args, **kwargs):
        """Perform an HTTP Request.

        You should use get, post, delete instead.

        """
        kwargs.setdefault('headers', kwargs.get('headers', {}))
        kwargs['headers']['User-Agent'] = self.USER_AGENT
        kwargs['headers']['Accept'] = 'application/json'
        if 'data' in kwargs:
            payload = kwargs['data']
        else:
            payload = None

        # args[0] contains the URL, args[1] contains the HTTP verb/method
        http_url = args[0]
        http_method = args[1]

        self._http_log_req(args, kwargs)
        r = None
        resp = None
        body = None
        while r is None and self.tries > 0:
            try:
                # Check to see if the request is being retried. If it is, we
                # want to delay.
                if self.delay:
                    time.sleep(self.delay)

                if self.timeout:
                    r = requests.request(http_method, http_url, data=payload,
                                         headers=kwargs['headers'],
                                         auth=self.auth,
                                         verify=self.secure,
                                         timeout=self.timeout)
                else:
                    r = requests.request(http_method, http_url, data=payload,
                                         auth=self.auth,
                                         headers=kwargs['headers'],
                                         verify=self.secure)

                resp = r.headers
                body = r.text
                if isinstance(body, bytes):
                    body = body.decode('utf-8')

                # resp['status'], status['content-location'], and resp.status
                # need to be manually set as Python Requests doesn't provide
                # them automatically.
                resp['status'] = str(r.status_code)
                resp.status = r.status_code
                if 'location' not in resp:
                    resp['content-location'] = r.url

                r.close()
                self._http_log_resp(resp, body)

                # Try and convert the body response to an object
                # This assumes the body of the reply is JSON
                if body:
                    try:
                        body = json.loads(body)
                    except ValueError:
                        pass
                else:
                    body = None

                if resp.status >= 400:
                    if body and 'message' in body:
                        body['desc'] = body['message']

                    raise exceptions.from_response(resp, body)
            except requests.exceptions.SSLError as err:
                self._logger.error(
                    "SSL certificate verification failed: (%s). You must have "
                    "a valid SSL certificate or disable SSL "
                    "verification.", err)
                raise exceptions.SSLCertFailed(
                    "SSL Certificate Verification Failed.")
            except self.retry_exceptions as ex:
                # If we catch an exception where we want to retry, we need to
                # decrement the retry count prepare to try again.
                r = None
                self.tries -= 1
                self.delay = self.delay * self.backoff + 1

                # Raise exception, we have exhausted all retries.
                if self.tries is 0:
                    raise ex
            except requests.exceptions.HTTPError as err:
                raise exceptions.HTTPError("HTTP Error: %s" % err)
            except requests.exceptions.URLRequired as err:
                raise exceptions.URLRequired("URL Required: %s" % err)
            except requests.exceptions.TooManyRedirects as err:
                raise exceptions.TooManyRedirects(
                    "Too Many Redirects: %s" % err)
            except requests.exceptions.Timeout as err:
                raise exceptions.Timeout("Timeout: %s" % err)
            except requests.exceptions.RequestException as err:
                raise exceptions.RequestException(
                    "Request Exception: %s" % err)
        return resp, body

    def _time_request(self, url, method, **kwargs):
        start_time = time.time()
        resp, body = self.request(url, method, **kwargs)
        self.times.append(("%s %s" % (method, url),
                           start_time, time.time()))
        return resp, body

    def _cs_request(self, url, method, **kwargs):
        resp, body = self._time_request(self.api_url + url, method,
                                        **kwargs)
        return resp, body

    def get(self, url, **kwargs):
        return self._cs_request(url, 'GET', **kwargs)

    def post(self, url, **kwargs):
        return self._cs_request(url, 'POST', **kwargs)

    def put(self, url, **kwargs):
        return self._cs_request(url, 'PUT', **kwargs)

    def delete(self, url, **kwargs):
        return self._cs_request(url, 'DELETE', **kwargs)

    def get_api(self):
        """Get the API endpoints."""
        return self.get("/api")

    def get_config(self):
        """Get the complete config object."""
        return self.get("/api/config")

    def get_gatewayinfo(self):
        """Get the number of active sessions on local gateway."""
        return self.get("/api/gatewayinfo")

    def get_targets(self):
        """Get the list of targets defined in the config."""
        api = "/api/targets"
        return self.get(api)

    def create_target_iqn(self, target_iqn, mode=None, controls=None):
        """Create the target iqn on the gateway."""
        api = "/api/target/%(target_iqn)s" % {'target_iqn': target_iqn}
        payload = {}
        if mode:
            payload['mode'] = mode

        if controls:
            payload['controls'] = controls

        return self.put(api, data=payload)

    def delete_target_iqn(self, target_iqn):
        """Delete a target iqn from the gateways."""
        api = "/api/target/%(target_iqn)s" % {'target_iqn': target_iqn}
        return self.delete(api)

    def get_clients(self, target_iqn):
        """List clients defined to the configuration."""
        api = "/api/clients/%(target_iqn)s" % {'target_iqn': target_iqn}
        return self.get(api)

    def get_client_info(self, target_iqn, client_iqn):
        """Fetch the Client information from the gateways.

        Alias, IP address and state for each connected portal.
        """
        api = ("/api/clientinfo/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        return self.get(api)

    def create_client(self, target_iqn, client_iqn):
        """Delete a client."""
        api = ("/api/client/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        return self.put(api)

    def delete_client(self, target_iqn, client_iqn):
        """Delete a client."""
        api = ("/api/client/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        return self.delete(api)

    def set_client_auth(self, target_iqn, client_iqn, username, password):
        """Set the client chap credentials."""
        url = ("/api/clientauth/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        args = {'username': username,
                'password': password}
        return self.put(url, data=args)

    def get_disks(self):
        """Get the rbd disks defined to the gateways."""
        return self.get("/api/disks")

    def create_disk(self, pool, image, size=None, extras=None):
        """Add a disk to the gateway."""
        url = ("/api/disk/%(pool)s/%(image)s" %
               {'pool': pool,
                'image': image})
        args = {'pool': pool,
                'image': image,
                'mode': 'create'}
        if size:
            args['size'] = size

        if extras:
            args.update(extras)
        return self.put(url, data=args)

    def find_disk(self, pool, image):
        """Find the disk in the gateway."""
        url = ("/api/disk/%(pool)s/%(image)s" %
               {'pool': pool,
                'image': image})
        return self.get(url)

    def delete_disk(self, pool, image):
        """delete a disk from the gateway."""
        url = ("/api/disk/%(pool)s/%(image)s" %
               {'pool': pool,
                'image': image})
        return self.delete(url)

    def register_disk(self, target_iqn, volume):
        """Add the volume to the target definition.

        This is done after the disk is created in a pool, and
        before the disk can be exported to an initiator.
        """
        url = ("/api/targetlun/%(target_iqn)s" %
               {'target_iqn': target_iqn})
        args = {'disk': volume}
        return self.put(url, data=args)

    def unregister_disk(self, target_iqn, volume):
        """Remove the volume from the target definition.

        This is done after the disk is unexported from an initiator
        and before the disk can be deleted from the gateway.
        """
        url = ("/api/targetlun/%(target_iqn)s" %
               {'target_iqn': target_iqn})
        args = {'disk': volume}
        return self.delete(url, data=args)

    def export_disk(self, target_iqn, client_iqn, pool, disk):
        """Add a disk to export to a client."""
        url = ("/api/clientlun/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        args = {'disk': "%(pool)s/%(disk)s" % {'pool': pool, 'disk': disk},
                'client_iqn': client_iqn}
        return self.put(url, data=args)

    def unexport_disk(self, target_iqn, client_iqn, pool, disk):
        """Remove a disk to export to a client."""
        url = ("/api/clientlun/%(target_iqn)s/%(client_iqn)s" %
               {'target_iqn': target_iqn,
                'client_iqn': client_iqn})
        args = {'disk': "%(pool)s/%(disk)s" % {'pool': pool, 'disk': disk}}
        return self.delete(url, data=args)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import pbr.version

# -- General configuration ---------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.viewcode',
              'reno.sphinxext',
              'pbr.sphinxext',
              'openstackdocstheme']

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'rbd-iscsi-client'
copyright = u"2019, Walter A. Boring IV"
author = u"Walter A. Boring IV"

# The version info for the project you're documenting, acts as replacement
# for |version| and |release|, also used in various other places throughout
# the built documents.
#
# The short X.Y version.
version_info = pbr.version.VersionInfo('rbd-iscsi-client')
version = version_info.version_string()
release = version_info.release_string()

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

add_function_parentheses = True
add_module_names = True

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output -------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'openstackdocs'

html_last_updated_fmt = '%Y-%m-%d %H:%M'


# -- Options for HTMLHelp output ---------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'rbd_iscsi_clientdoc'


# -- Options for LaTeX output ------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'rbd_iscsi_client.tex',
     u'RBD iSCSI Client Documentation',
     u'Walter A. Boring IV', 'manual'),
]


# -- Options for manual page output ------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'rbd_iscsi_client',
     u'RBD iSCSI Client Documentation',
     [author], 1)
]


# -- Options for Texinfo output ----------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'rbd_iscsi_client',
     u'RBD iSCSI Client Documentation',
     author,
     'rbd_iscsi_client',
     'One line description of project.',
     'Miscellaneous'),
]


# -- Options for openstackdocstheme -----------------------------------
repository_name = 'hemna/rbd-iscsi-client'
bug_project = 'rbd-iscsi-client'
bug_tag = ''
